// Programmer: Maura Dailey
// CSE 120 Section: 1
// Date: 04/26/02

//  Brief: This file includes the function definitions for the file theatre.h
//    and provides clarifying commentary as well as being the backbone for the
//    EZ Theatre Manager program.


#include <iostream>
#include <fstream>
#include <iomanip>
#include <list>
#include <vector>
#include <string>

Theatre::Theatre()
{
  // Open the file Theater in order to initialize the size of the theatre
  ifstream setup;
  setup.open("Theater");
  theatre.resize(100);

  int temp_row, temp_seat;

  // Read the values for the size of the theatre, assigning a length to each
  // row individually

  while (!setup.eof())
  {
    setup >> temp_row;
    setup >> temp_seat;

    theatre[(temp_row-1)] = row(temp_seat);
  }

  // Resize the theatre and close the input file
  theatre.resize(temp_row);
  setup.close();

  // Open the file Reserved, which may or may not contain data from previous
  // program executions

  ifstream reserved;
  reserved.open("Reserved");
  
  // If the file has opened successfully, begin to read data and fill the theatre
  if (reserved.is_open())
  {
    char temp;

    reserved >> temp;
    reserved.ignore();

    while(!reserved.eof())
    {
      // A value of S indicates that the data on that line belongs in
      // the list of orders

      if (temp == 'S')
      {
        Order new_order;
        getline(reserved,new_order.name,'$');
        reserved >> new_order.num_tickets;
        reserved >> new_order.begin_seat;
        reserved >> new_order.begin_row;

        // Once data has been read and assigned to a new order struct, add
        // it to the list of orders

        order_list.push_back(new_order);

        int num_tickets = (new_order.num_tickets);
        int seats = (new_order.begin_seat);
        int row = (new_order.begin_row);

        // Assign seats in the theatre to accomodate this new order

        assignSeats(theatre, num_tickets, row, seats);

        reserved >> temp;
        reserved.ignore();
        if (reserved.eof())
          break;
      }

      // A value of W indicates that the data on that line should be
      // assigned to a waitlist

      else if (temp == 'W')
      {
        Waitlist waitlist;
        getline(reserved,waitlist.name,'$');
        reserved >> waitlist.num_tickets;

        // Once data has been read and assigned to a new waitlist struct, add
        // the data to the waitlist

        wait_list.push_back(waitlist);
        reserved >> temp;
        reserved.ignore();
        if (reserved.eof())
          break;
      }

      else
      {
        // The file reserved must be corrupted, warn the user that an error has
        // occurred.

        cout << "File corruption" << endl;
      }
    }
  
    reserved.close();
  }

  // Open the transactions file and input a short descriptory message

  transactions.open("Reserved.trn");
  transactions << "Welcome to the EZ Theatre Manager program. The contents of this"
               << endl << "file are a listing of the transactions that occurred the last"
               << endl << "time the program was executed."  << endl << endl;
  transactions << "Transaction file begun. Now printing program output:" << endl;
  transactions.close();
}

void Theatre::addOrder(string name, int num_tickets)
{
  // Create a new order struct
  Order new_order;

  // Try to find available seats in the theatre
  findSeats(theatre, num_tickets, rowNumber, seatNumber);

  // If seats were found, assign the data to an order and add it to the list of orders, then
  // assign the correct number of seats

  if (rowNumber != 0)
  {
    new_order.name = name;
    new_order.num_tickets = num_tickets;
    new_order.begin_row = rowNumber;
    new_order.begin_seat = seatNumber;

    // Provide explanatory output
    transactions.open("Reserved.trn", ios::app);
    cout << endl << "ORDER " << name << endl;
    transactions << endl << "ORDER " << name << endl;

    // Pad the length of name for better formatting
    while (name.length() < 19)
    {
      name+=" ";
    }

    // Detect whether the order is one ticket or not, and if so, modify output accordingly
    if ((seatNumber+num_tickets-1) == seatNumber)
    {
      // Output order receipt to screen
      cout << "  " << name << setw(3) << num_tickets << setw(20) << " seat   Reserving Row "
           << rowNumber << ", Seat  " << setw(3) << seatNumber << endl;

      // Output order receipt to transaction file
      transactions << "  " << name << setw(3) << num_tickets << setw(20) << " seat   Reserving Row "
                   << rowNumber << ", Seat  " << setw(3) << seatNumber << endl;
    }

    else
    {
      // Output order receipt to screen
      cout << "  " << name << setw(3) << num_tickets << setw(20) << " seats  Reserving Row "
           << rowNumber << ", Seats " << setw(3) << seatNumber << setw(9) << "through"
           << setw(3) << (seatNumber+num_tickets-1) << endl;

      // Output order receipt to transaction file
      transactions << "  " << name << setw(3) << num_tickets << setw(20) << " seats  Reserving Row "
                   << rowNumber << ", Seats " << setw(3) << seatNumber << setw(9) << "through"
                   << setw(3) << (seatNumber+num_tickets-1) << endl;
    }

    // Add order to list of orders
    order_list.push_back(new_order);
    transactions.close();

    // Assign seats to new order
    assignSeats(theatre, num_tickets, rowNumber, seatNumber);
  }

  // If seats cannot be found, add to waitlist
  else
  {
    addToWaitlist(name, num_tickets);
  }
}

void Theatre::addToWaitlist(string name, int num_tickets)
{
  // Open the transactions file for output and create a waitlist struct
  transactions.open("Reserved.trn", ios::app);
  Waitlist will_call;

  will_call.name = name;
  will_call.num_tickets = num_tickets;

  // Add new waitlist item to waitlist
  wait_list.push_back(will_call);

  cout << endl << "ORDER " << name << endl;
  transactions << endl << "ORDER " << name << endl;

  // Increase the length of name for better output
  while(name.length() < 19)
  {
    name+=" ";
  }

  // If the order only contains one ticket, adjust output accordingly
  if (num_tickets == 1)
  {
    // Output waitlist receipt to screen
    cout << "  " << name << setw(3) << num_tickets << setw(9) << "seat  "
         << "This order has been put on the Wait List." << endl;

    // Output waitlist receipt to transactions file
    transactions << "  " << name << setw(3) << num_tickets << setw(9) << "seat  "
                 << "This order has been put on the Wait List." << endl;
  }

  else
  {
    // Output waitlist receipt to screen
    cout << "  " << name << setw(3) << num_tickets << setw(9) << " seats "
         << "This order has been put on the Wait List." << endl;

    // Output waitlist receipt to transactions file
    transactions << "  " << name << setw(3) << num_tickets << setw(9) << " seats "
                 << "This order has been put on the Wait List." << endl;
  }

  transactions.close();
}

void Theatre::addFromWaitlist()
{
  transactions.open("Reserved.trn", ios::app);
  list<Waitlist>::iterator iter;

  // Search the waitlist and attempt to find seats for each waitlist item
  for(iter = wait_list.begin(); iter != wait_list.end(); iter++)
  {
    int num_tickets=((*iter).num_tickets);
    string name = ((*iter).name);

    // Attempt to find seats for each waitlist item
    findSeats(theatre, num_tickets, rowNumber, seatNumber);

    // If seats are found, proceed to assign the current waitlist item to
    // them and add the waitlist item to the list of orders

    if(rowNumber != 0)
    {
      Order new_order;

      new_order.name = name;
      new_order.num_tickets = num_tickets;
      new_order.begin_row = rowNumber;
      new_order.begin_seat = seatNumber;

      // Display that you are reserving seats for the patron, and display full item
      // detail about the order

      if (seatNumber == (seatNumber+num_tickets-1))
      {
        cout << "  " << setw(12) << "Reserving" << setw(5) << "Row " << setw(4)
             << rowNumber << setw(7) << "Seat" << setw(3) << seatNumber << "for "
             << name << endl;

        transactions << "  " << setw(12) << "Reserving" << setw(5) << "Row " << setw(4)
                     << rowNumber << setw(7) << "Seat" << setw(3) << seatNumber << "for "
                     << name << endl;
      }

      else
      {
        cout << "  " << setw(12) << "Reserving" << setw(5) << "Row " << setw(4)
             << rowNumber << setw(7) << "Seats" << setw(3) << seatNumber << setw(9)
             << "through" << setw(3) << (seatNumber+num_tickets-1) << "for "
             << name << endl;
             
        transactions << "  " << setw(12) << "Reserving" << setw(5) << "Row " << setw(4)
                     << rowNumber << setw(7) << "Seats" << setw(3) << seatNumber << setw(9)
                     << "through" << setw(3) << (seatNumber+num_tickets-1) << "for "
                     << name << endl;
      }

      // Add the newly created order to the list of orders
      order_list.push_back(new_order);

      // Assign seats for the new order
      assignSeats(theatre, num_tickets, rowNumber, seatNumber);

      // Delete the order from the waitlist
      deleteFromWaitlist(name);
    }

    // Begin again at the beginning of the waitlist
    iter = wait_list.begin();
  }

  transactions.close();
}

void Theatre::deleteOrder(string name)
{
  transactions.open("Reserved.trn", ios::app);

  // Create iterators for both the list of orders and the waitlist
  list<Order>::iterator iter;
  list<Waitlist>::iterator iter2;
  
  cout << endl << "CANCEL " << name << endl;
  transactions << endl << "CANCEL " << name << endl;

  // Move through the entire list of orders and delete every instance of the
  // patron's orders

  for (iter=order_list.begin(); iter != order_list.end(); iter++)
  {
    if(((*iter).name) == name)
    {
      int num_tickets=((*iter).num_tickets);
      rowNumber=((*iter).begin_row);
      seatNumber=((*iter).begin_seat);

      // Unassign seats to free them for other patrons
      unassignSeats(theatre, num_tickets, rowNumber, seatNumber);

      // Inform the user which seats are being freed by the deletion of the patron's
      // orders

      if(seatNumber == (seatNumber+num_tickets-1))
      {
        cout << "  " << setw(12) << "Freeing" << setw(5) << "Row " << setw(4)
             << rowNumber << setw(7) << "Seat" << setw(3) << seatNumber
             << endl;

        transactions << "  " << setw(12) << "Freeing" << setw(5) << "Row " << setw(4)
                     << rowNumber << setw(7) << "Seat" << setw(3) << seatNumber
                     << endl;
      }

      else
      {
        cout << "  " << setw(12) << "Freeing" << setw(5) << "Row " << setw(4)
             << rowNumber << setw(7) << "Seats" << setw(3) << seatNumber << setw(9)
             << "through" << setw(3) << (seatNumber+num_tickets-1) << endl;

        transactions << "  " << setw(12) << "Freeing" << setw(5) << "Row " << setw(4)
                     << rowNumber << setw(7) << "Seats" << setw(3) << seatNumber << setw(9)
                     << "through" << setw(3) << (seatNumber+num_tickets-1) << endl;
      }

      // Delete the patron from the list of orders and begin to search through the list again
      order_list.erase(iter);
      iter = order_list.begin();
    }
  }

  // Search through the waitlist for instances of the patron's orders and delete any that are
  // found

  for (iter2 = wait_list.begin(); iter2 != wait_list.end(); iter2++)
  {
    if(((*iter2).name)==name)
    {
      int num_tickets=((*iter2).num_tickets);
      
      wait_list.erase(iter2);
      iter2 = wait_list.begin();
    }
  }

  transactions.close();
  
  // Add new orders from the waitlist to fill any newly available seats
  addFromWaitlist();
}

void Theatre::deleteFromWaitlist(string name)
{
  // Search through the waitlist and delete any instances of the patron's
  // name

  list<Waitlist>::iterator iter;
  for (iter = wait_list.begin(); iter != wait_list.end(); iter++)
  {
    if(((*iter).name)==name)
    {
      int num_tickets=((*iter).num_tickets);
      
      // Delete the patron from the waitlist
      wait_list.erase(iter);
      iter = wait_list.begin();
    }
  }
}

void Theatre::print()
{
  // Reopen the transactions file and set it to append new output
  transactions.open("Reserved.trn", ios::app);
  
  // Create iterators for both the list of orders and the waitlist
  list<Order>::const_iterator iter;
  list<Waitlist>::const_iterator iter2;
  
  // Print descriptive output to the screen and to the transactions file

  cout << endl << "STATUS:" << endl << "  Reservations:" << endl;
  transactions << endl << "STATUS:" << endl << "  Reservations:" << endl;

  // Move through the entire list of orders and print each order detail to the screen
  // and to the transactions file

  for (iter=order_list.begin(); iter != order_list.end(); iter++)
  {
    
    // For better output, output has been customized to correctly recognize if only
    // one seat is in an order or whether there is a set of seats.

    if ((((*iter).begin_seat)+((*iter).num_tickets)-1) == ((*iter).begin_seat))
    {
      string name = ((*iter).name);

      // To correct formatting and provide a nicely lined up table, it is necessary to pad
      // the name to the correct length for output purposes.

      while (name.length() < 19)
      {
        name+=" ";
      }

      // Write the order details to the screen

      cout << "    " << name << setiosflags(ios::left) << setw(5) << "Row" << setw(7)
           << ((*iter).begin_row) << setw(7) << "Seat" << setw(3) << ((*iter).begin_seat)
           << endl;

      // Write the order details to the transaction file

      transactions << "    " << name << setiosflags(ios::left) << setw(5) << "Row" << setw(7)
                   << ((*iter).begin_row) << setw(7) << "Seat" << setw(3) << ((*iter).begin_seat)
                   << endl;
    }

    else
    {
      string name = ((*iter).name);

      // To correct formatting and provide a nicely lined up table, it is necessary to pad
      // the name to the correct length for output purposes.

      while (name.length() < 19)
      {
        name+=" ";
      }
      
      // Write the order detail to the screen

      cout << "    " << name << setiosflags(ios::left) << setw(5) << "Row" << setw(7)
           << ((*iter).begin_row) << setw(7) << "Seats" << setw(3) << ((*iter).begin_seat)
           << setw(9) << "through" << setw(3) << (((*iter).begin_seat)+((*iter).num_tickets)-1)
           << endl;

      // Write the order detail to the transactions file

      transactions << "    " << name << setiosflags(ios::left) << setw(5) << "Row" << setw(7)
                   << ((*iter).begin_row) << setw(7) << "Seats" << setw(3) << ((*iter).begin_seat)
                   << setw(9) << "through" << setw(3) << (((*iter).begin_seat)+((*iter).num_tickets)-1)
                   << endl;
    }
  }

  iter = order_list.begin();

  // If the beginning of the list equals the end, then the list must be empty. Inform the user.

  if (iter == order_list.end())
  {
    cout << "    Reservation list is empty." << endl;
    transactions << "    Reservation list is empty." << endl;
  }

  // Repeat the process of outputting each item detail, but this time for the contents of the wait
  // list.

  cout << endl << "  Waiting List:" << endl;
  transactions << endl << "  Waiting List:" << endl;

  // Traverse through the waitlist and display each item detail to the screen and to a file

  for (iter2=wait_list.begin(); iter2 != wait_list.end(); iter2++)
  {
    // Pad the patron's name for better formatting to the screen and to the transactions file

    string name = ((*iter2).name);
    while (name.length() < 19)
    {
      name+=" ";
    }

    cout << "    " << name << setw(3) << ((*iter2).num_tickets) << "seats" << endl;
    transactions << "    " << name << setw(3) << ((*iter2).num_tickets) << "seats" << endl;
  }

  iter2 = wait_list.begin();

  // If the beginning of the waitlist equals the end of the waitlist, the waitlist must be empty.
  // Inform the user.

  if(iter2 == wait_list.end())
  {
    cout << "    Waiting list is empty." << endl;
    transactions << "    Waiting list is empty." << endl;
  }

  cout << endl;
  transactions.close();
}

void Theatre::findSeats(vector<row>& theatre, int count, int& rowNumber, int& seatNumber)
{
  // Initialize helper variables
  int i, j, tcount;

  // Search the entire two dimensional vector for a block of open seats, and set rowNumber and
  // seatNumber to be the proper starting position to begin assigning seats

  for (i = 0; i < theatre.size(); i++)
  {
    tcount = count;
    for (j = 0; j < theatre[i].size(); j++)
    {
      if(!theatre[i][j])
      {
        tcount--;
        if(tcount == 0)
        {
          rowNumber = i + 1;
          seatNumber = j - count + 2;
          return;
        }
      }
      else
      {
        tcount = count;
      }
    }
  }

  // If there are no open seats, return zero values
  rowNumber = 0;
  seatNumber = 0;
}

void Theatre::assignSeats(vector< row >& theatre, int count, int& rowNumber, int& seatNumber)
{
  // Begin at rowNumber and seatNumber and assign a value to as many seats as are defined with
  // count to fill the seats of the theatre.

  int i;
  for (i=seatNumber - 1; i < seatNumber + count - 1; i++)
  {
    theatre[rowNumber - 1][i] = true;
  }
}

void Theatre::unassignSeats(vector< row >& theatre, int count, int& rowNumber, int& seatNumber)
{
  // Begin at rowNumber and seatNumber and empty as many seats as are defined with count

  int i;
  for (i=seatNumber - 1; i < seatNumber + count - 1; i++)
  {
    theatre[rowNumber - 1][i] = false;
  }
}

void Theatre::writeReserved()
{
  ofstream reserved;
  reserved.open("Reserved");
  
  list<Order>::const_iterator iter;
  list<Waitlist>:: const_iterator iter2;

  for (iter=order_list.begin(); iter != order_list.end(); iter++)
  {
    reserved << "S " << ((*iter).name) << "$" << ((*iter).num_tickets) << " " 
             << ((*iter).begin_seat) << " " << ((*iter).begin_row) << endl;
  }
  
  for (iter2=wait_list.begin(); iter2 != wait_list.end(); iter2++)
  {
    reserved << "W " << ((*iter2).name) << "$" << ((*iter2).num_tickets) << endl;
  }
  
  reserved << endl;
  reserved.close();
  
  transactions.open("Reserved.trn", ios::app);
  
  transactions << endl << "Thank you for using the EZ Theatre Manager! The status of"
               << endl << "your theatre has been saved to a file and records of your"
               << endl << "transactions have been saved to Reserved.trn."
               << endl;
  
  transactions.close();
}
