// Programmer: Maura Dailey
// CSE 120 Section #: 1
// Date: 04/26/02

// Brief: This file determines the function prototypes used by the EZ
//   Theatre Manager

#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <queue>
#include <vector>

typedef vector<bool> row;

class Theatre
{
  public:

    Theatre();
    // A constructor which creates the theatre and initializes it, then
    // fills it according to the dictates of Reserved

    void addOrder(string name, int num_tickets);
    // Take name and num_tickets as arguments. Search for empty rows.
    // If rows are found, assign to list, else assign to waitlist.
    // Store data in a list of orders.
    
    void addToWaitlist(string name, int num_tickets);
    // Take name and num_tickets as arguments. Add values to waitlist
    // to await an opening in the theatre seating.

    void addFromWaitlist();
    // Query each item in the waitlist and attempt to find seats for
    // each. If seats are found, assign them and add the member to the
    // list of orders, deleting it from the waitlist.

    void deleteOrder(string name);
    // Take the name of the patron, search for all instances and delete
    // each order associated with that name, be it in the list of orders
    // or in the waitlist. Call addFromWaitlist() when done to attempt to
    // fill the newly emptied seats.

    void deleteFromWaitlist(string name);
    // Specialized function to delete a patron from a waitlist once they
    // have been added to the list of orders

    void print();
    // Print status output to the screen and to a transaction file by listing
    // each item in order in both the list of orders and the waitlist

    void findSeats(vector<row>& theatre, int count, int& rowNumber, int& seatNumber);
    // Search the theatre for available seating. If seating is found, assign RowNumber
    // and seatNumber to the appropriate place to begin assigning seats. Otherwise return
    // 0's for both variables.
    
    void assignSeats(vector<row>& theatre, int count, int& rowNumber, int& seatNumber);
    // Assign a block of seats with the specified length beginning at rowNumber and
    // seatNumber.

    void unassignSeats(vector<row>& theatre, int count, int& rowNumber, int& seatNumber);
    // Unassign a block of seats of the specified length beginning at rowNumber and
    // seatNumber
    
    void writeReserved();
    // Write the contents of Reserved, in order to later restore the seating arrangement of
    // the theatre on the next execution.

  private:
    // A struct data type that can hold data about an order
    struct Order
    {
      string name;
      int num_tickets;
      int begin_row;
      int begin_seat;
    };

    // A struct data type that can hold data about a waitlist item
    struct Waitlist
    {
      string name;
      int num_tickets;
    };

    // Used to track where seating may be found in the theatre
    int rowNumber;
    int seatNumber;

    // A list of orders and a waitlist respectively
    list<Order> order_list;
    list<Waitlist> wait_list;
    
    // The theatre itself
    vector< row > theatre;

    // The transactions file
    ofstream transactions;
};

#include "Theatre.cpp"
