// Programmer: Maura Dailey

//  Brief: This program, which I have entitled the EZ Theatre Manager
//    program, is designed to allow users to interactively add, remove
//    and view orders placed for seats in a theatre. Users decide what
//    they wish to do by following a menu system. Orders are stored in
//    the system by the name of the person placing the order.

//  Output: Results of all transactions are displayed to the screen, and
//    then stored to a separate output file titled Reserved.trn.
//    Additionally, the current state of the theatre is stored to an
//    output file titled Reserved that will be read and implemented at
//    startup.
//
//  Input: Input comes from Theater, which is a user defined file that
//    determines the size of the theatre. Other input comes in the form
//    of a file entitled Reserved that will store any previous theatre
//    state.

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include "Theatre.h"

int main()
{
  using namespace std;

  // Initialize variables, create class
  Theatre theatre;
  char command;
  string name;
  int ticket_num;

  // Initial output for the interactive menubased data entering system

  cout << endl << "Welcome to the EZ Theatre Manager program!" << endl;
  cout << "Please choose a command from the menu below:" << endl;

  cout << endl << "1. Place an order"
       << endl << "2. Delete an existing order"
       << endl << "3. Print current theatre status"
       << endl << "4. Exit program and save current state"
       << endl << "5. See this menu again"
       << endl << endl;

  // As long as the user does not enter 4, which demonstrates that the user desires
  // to exit the program, continue to accept input and process output to the screen

  while (command != '4')
  {
    // Prompt the user to choose a command
    cout << "Enter a command from 1 - 4 or enter 5 to see the menu again:" << endl;
    cout << "> ";
    cin >> command;
    cin.ignore();

    // Determine the command chosen and choose an appropriate branch according to the
    // recommended plan of action outlined by the menu system

    switch (command)
    {
      case '1':  cout << endl << "Now creating a new order:" << endl;
                 cout << "Enter name of patron placing order:" << endl;
                 cout << "> ";
                 getline(cin, name);

                 cout << endl;
                 cout << "Enter number of tickets patron wishes to order:" << endl;
                 cout << "> ";
                 cin >> ticket_num;

                 // Once user input has been recorded, call the addOrder() function
                 // to add this order to the order list. It will automatically detect
                 // if the order is too large, and if so, will add it instead to a
                 // waitlist that will be read each time an order is canceled.

                 theatre.addOrder(name, ticket_num);
                 cout << endl;
                 break;

      case '2':  cout << endl << "Now deleting existing orders:" << endl << endl;
                 cout << "Enter name of patron:" << endl;
                 cout << "> ";
                 getline(cin, name);

                 // Once the user has determined what order they wish to delete,
                 // deleteOrder() is called. Contents of both the list of orders
                 // and the waitlist are searched for the name of the patron
                 // canceling the order. Additionally, if deleting these orders
                 // opens up new space, orders are added in consecutive order
                 // from the waitlist

                 theatre.deleteOrder(name);
                 cout << endl;
                 break;

      case '3':  theatre.print();
      
                 // Print the contents of both the list of orders and the current
                 // waitlist.

                 break;
      
      case '4':  break;
      
                 // User has chosen to quit program.
      
      default:   cout << endl << "Please choose a command from the menu below:" << endl;

                 cout << endl << "1. Place an order"
                      << endl << "2. Delete an existing order"
                      << endl << "3. Print current theatre status"
                      << endl << "4. Exit program and save current state"
                      << endl << "5. See this menu again"
                      << endl << endl;

                 // User has chosen to refresh the menu to the screen

                 break;
      }
  }
  
  cout << endl << "Thank you for using the EZ Theatre Manager! The status of"
       << endl << "your theatre has been saved to a file and records of your"
       << endl << "transactions have been saved to Reserved.trn."
       << endl << endl;

  // Display final remarks to the screen and write the current state of the theatre to a
  // file titled Reserved which will let the state of the theater be restored at a later
  // date.

  theatre.writeReserved();
  
  return 0;
}
